# -*- coding: utf-8 mode: python -*- vim:sw=4:sts=4:et:ai:si:sta:fenc=utf-8

__all__ = (
    'checked', 'selected', 'disabled', 'required',
    'accesskeyif',
    'classif', 'addclassif',
    'favicon', 'css', 'js', 'jscss',
    'u', 'p',
    'Action', 'Menu',
)

from types import StringType, UnicodeType
import urlparse, urllib

from nulib.base import odict, isseq, seqof
from nulib.web import web

def checked(b):
    return u' checked="checked"' if b else u''
def selected(b):
    return u' selected="selected"' if b else u''
def disabled(b):
    return u' disabled="disabled"' if b else u''
def required(b):
    return u' required="required"' if b else u''

def accesskeyif(ak, b=None):
    if b is None: b = ak
    return u' accesskey="%s"' % ak if b else u''

def classif(c, b=None):
    if b is None: b = c
    return u' class="%s"' % c if b else u''
def addclassif(c, b=None, prefix=None):
    if b is None: b = c
    if prefix is None: prefix = u''
    if b: return u'%s %s' % (prefix, c)
    else: return prefix

def favicon(href):
    return ur'<link rel="shortcut icon" href="%s" />' % href
def css(href, media=None):
    if media is not None:
        media = ur' media="%s"' % media
    return ur'<link rel="stylesheet" href="%s" type="text/css"%s />' % (href, media or u'')
def js(href):
    return ur'<script type="text/javascript" src="%s"></script>' % href
def jscss(href, min=False):
    if href.endswith(".min.js"):
        jshref = href
        csshref = "%s.css" % href[:-len(".min.js")]
    elif href.endswith(".js"):
        jshref = href
        csshref = "%s.css" % href[:-len(".js")]
    elif href.endswith(".css"):
        jshref = "%s.js" % href[:-len(".css")]
        csshref = href
    return u"%s\n%s" % (js(jshref), css(csshref))

def u(text):
    """Si text n'est pas de l'unicode, le convertir en unicode avec l'encoding utf-8
    """
    t = type(text)
    if t is not UnicodeType:
        if t is not StringType: text = str(text)
        text = unicode(text, "utf-8")
    return text

def p(text):
    """Si text commence par un tag, le laisser tel quel, sinon le mettre entre <p> et </p>
    """
    text = u(text)
    if text[0:1] == '<': return text
    else: return u'<p>%s</p>' % text

################################################################################
# Menus

class Action(odict):
    """Un lien ou une action d'un formulaire

    url: cible de l'action ou du lien (avec querystring)
    title: titre du lien ou de l'élément de menu
    id: identifiant dans un menu
    p (in_profiles): profils dans lequel l'action est valide. le premier profil
        est le profil par défaut
    c (css): classe css à appliquer au lien
    ak (accesskey): accesskey à installer sur le lien
    q (query): paramètres de la requête, sous forme de dictionnaire ou de chaine
    method: type de requête, get ou post
    PROFILE_url, PROFILE_query: valeurs spécifiques à certains profils, PROFILE
        étant une valeur de in_profiles
    to_PROFILE_url, to_PROFILE_query: valeurs spécifiques pour basculer vers le
        profil spécifié, PROFILE étant une valeur de in_profiles

    les valeurs *url et *query sont reconstruites: baseurl et basequery sont
    extrait d'url et query est reconstruit en le fusionnant avec basequery

    Les valeurs suivantes sont générées:
    baseurl: cible de l'action ou du lien (sans querystring)
    """

    def __merge_query(self, query, basequery):
        if query is None: query = {}
        elif not isinstance(query, dict):
            query = urlparse.parse_qs(query, True)
        return dict(basequery, **query)
    def __update_url(self, urlkey, querykey, baseurlkey):
        url = self.get(urlkey, None)
        if url is None: return
        urlsplit = urlparse.urlsplit(url)
        baseurl = urlparse.urlunsplit(urlsplit[:3] + ('',) + urlsplit[4:])
        basequery = urlparse.parse_qs(urlsplit[3], True)
        query = self.get(querykey, None)
        query = self.__merge_query(query, basequery)
        url = urlparse.urlunsplit(
            urlsplit[:3]
            + (urllib.urlencode(query, True),)
            + urlsplit[4:])
        self[baseurlkey] = baseurl
        self[querykey] = query
        self[urlkey] = url

    def __init__(self, url, title=None, id=None, p=None, c=None, ak=None, q=None, method=None, **kw):
        super(Action, self).__init__(**kw)
        if isseq(url):
            if url[1:2]: title = url[1]
            if url[2:3]: id = url[2]
            if url[3:4]: p = url[3]
            if url[4:5]: c = url[4]
            if url[5:6]: ak = url[5]
            if url[6:7]: method = url[6]
            url = url[0] if url[0:1] else None
        if url is None: raise ValueError("url is required")
        if title is None: title = url
        if p is None: p = kw.get('in_profiles', None)
        if c is None: c = kw.get('css', None)
        if ak is None: ak = kw.get('accesskey', None)
        if q is None: q = kw.get('query', None)
        if method is None: method = 'get'

        self.title = title
        self.id = id
        self.css = c
        self.accesskey = ak
        self.method = method

        self.url = url
        self.query = q
        self.__update_url('url', 'query', 'baseurl')

        self.in_profiles = seqof(p, None)
        if self.in_profiles is None: return
        for p in self.in_profiles:
            self.__update_url(*['%s_%s' % (p, key) for key in ('url', 'query', 'baseurl')])
            self.__update_url(*['to_%s_%s' % (p, key) for key in ('url', 'query', 'baseurl')])

    def __get_key(self, key, p=None, to=False):
        if p is not None:
            pkey = '%s%s_%s' % ('to_' if to else '', p, key)
            value = self.get(pkey, None)
            if value is not None: return value
        return self[key]
    def get_baseurl(self, p=None, to=False):
        """obtenir l'url de base
        """
        return self.__get_key('baseurl', p, to)
    def get_query(self, p=None, to=False):
        """obtenir les paramètres de la requête sous forme de dictionnaire
        """
        return self.__get_key('query', p, to)
    def get_url(self, p=None, to=False):
        """obtenir l'url
        """
        return self.__get_key('url', p, to)

    def get_qs(self, p=None, to=False, sep=None):
        """obtenir les paramètres de la requête sous forme de query-string
        sep vaut par défaut '?' mais peut valoir '&'
        """
        query = self.get_query(p, to)
        if sep is None: sep = '?'
        return '%s%s' % (sep, urllib.urlencode(query, True))
    qs = property(get_qs)
    get_querystring = get_qs; querystring = property(get_qs)

    def get_inputs(self, p=None, to=False):
        """obtenir les paramètres de la requête sous forme d'une liste de définitions
        d'éléments de formulaire de type hidden
        """
        query = self.get_query(p, to)
        Hidden = web.form.Hidden
        inputs = []
        for name, value in query.items():
            if isinstance(value, list):
                for value in value:
                    inputs.append(Hidden(name=name, value=value).render())
            else:
                inputs.append(Hidden(name=name, value=value).render())
        return u"\n".join(inputs)
    inputs = property(get_inputs)

class Menu(odict):
    """Un menu"""

    nextid = None
    idmap = None

    def __nextid(self):
        nextid = self.nextid
        self.nextid += 1
        return nextid

    def __init__(self, title, mitems=None, profiles=None,
                 id=None, p=None, c=None, **kw):
        super(Menu, self).__init__(**kw)
        self.__dict__['nextid'] = 0
        self.__dict__['idmap'] = {}
        if title is None: title = u""
        if id is None: id = self.__nextid()
        self.title = title
        self.mitems = []
        profiles = seqof(profiles, None)
        self.profiles = profiles
        self.default_profile = profiles[0] if profiles is not None else None
        self.id = str(id)
        self.in_profiles = seqof(p, None)
        self.css = c
        self.sel_id = None
        self.sel_profile = None
        self.active = False
        for mitem in seqof(mitems):
            self.add(mitem)

    def add(self, mitem, *args, **kw):
        if isinstance(mitem, Menu): pass # sous-menu
        elif not isinstance(mitem, Action):
            mitem = Action(mitem, *args, **kw)
        if mitem.id is None:
            mitem.id = str(self.__nextid())
        #mitem.profiles = self.profiles
        mitem.default_profile = self.default_profile
        mitem.active = False
        self.mitems.append(mitem)
        self.idmap[mitem.id] = mitem

    def reset_selection(self):
        self.sel_id = None
        self.sel_profile = self.default_profile
        self.active = False
        for mitem in self.mitems:
            if isinstance(mitem, Menu):
                mitem.reset_selection()
            else:
                mitem.active = False

    def select(self, id, p=None):
        # d'abord déselectionner tout le monde
        self.reset_selection()
        # ensuite chercher le mitem à sélectionner
        id = str(id)
        mitem = self.idmap.get(id, None)
        if mitem is not None:
            self.sel_id = id
            if p is None and mitem.in_profiles:
                p = mitem.in_profiles[0]
            if p is None: p = self.default_profile
            self.sel_profile = p
            mitem.active = self.active = True
            return True
        else:
            for mitem in self.mitems:
                if not isinstance(mitem, Menu): continue
                if mitem.select(id):
                    self.sel_id = id
                    self.sel_profile = mitem.sel_profile
                    self.active = True
                    return True
            return False

    def get_mitem(self, id=None):
        """retourner l'élément de menu correspondant à la sélection courante
        """
        if id is None: id = self.sel_id
        if id is None: return None
        id = str(id)
        mitem = self.idmap.get(id, None)
        if mitem is not None: return mitem
        for mitem in self.mitems:
            if not isinstance(mitem, Menu): continue
            mitem = mitem.get_mitem(id)
            if mitem is not None: return mitem
        return None
    selection = property(get_mitem)
